/**
  ******************************************************************************
  * @file    spi_enc28j60.h
  * @author  YEES
  * @version V1.0.0
  * @date    2011/11/17
  * @brief   SPI functions ENC28J60. 
  *            
  *          +------------------------------------------+
  *          |             Pin assignment               |
  *          +-----------------------+------------------+
  *          |  STM32F4-Discovery    |     ENC28J60     |
  *          +-----------------------+------------------+
  *          | PB11                  |   RESET          |
  *          | PB12                  |   CS             |
  *          | PB13 / SPI2_SCLK      |   SCK            |
  *          | PB14 / SPI2_MISO      |   SO             |
  *          | PB15 / SPI2_MOSI      |   SI             |
  *          +-----------------------+------------------+
  ******************************************************************************
  * @attention
  * (C) COPYRIGHT 2011 YEES 
  ******************************************************************************  
  */ 

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __SPI_ENC28J60_H
#define __SPI_ENC28J60_H

#ifdef __cplusplus
 extern "C" {
#endif

/******************************************************************************/

//#define		USE_SOFTWAEW_SPI

/******************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"

/* Exported types ------------------------------------------------------------*/
/* Exported constants --------------------------------------------------------*/



  
/* M25P ENC28J60 SPI Interface pins  */  
#define spiENC28J60_SPI                           SPI2
#define spiENC28J60_SPI_CLK                       RCC_APB1Periph_SPI2
#define spiENC28J60_SPI_CLK_INIT                  RCC_APB1PeriphClockCmd

#define spiENC28J60_SPI_SCK_PIN                   GPIO_Pin_13
#define spiENC28J60_SPI_SCK_GPIO_PORT             GPIOB
#define spiENC28J60_SPI_SCK_GPIO_CLK              RCC_AHB1Periph_GPIOB
#define spiENC28J60_SPI_SCK_SOURCE                GPIO_PinSource13
#define spiENC28J60_SPI_SCK_AF                    GPIO_AF_SPI2

#define spiENC28J60_SPI_MISO_PIN                  GPIO_Pin_14
#define spiENC28J60_SPI_MISO_GPIO_PORT            GPIOB
#define spiENC28J60_SPI_MISO_GPIO_CLK             RCC_AHB1Periph_GPIOB
#define spiENC28J60_SPI_MISO_SOURCE               GPIO_PinSource14
#define spiENC28J60_SPI_MISO_AF                   GPIO_AF_SPI2

#define spiENC28J60_SPI_MOSI_PIN                  GPIO_Pin_15
#define spiENC28J60_SPI_MOSI_GPIO_PORT            GPIOB
#define spiENC28J60_SPI_MOSI_GPIO_CLK             RCC_AHB1Periph_GPIOB
#define spiENC28J60_SPI_MOSI_SOURCE               GPIO_PinSource15
#define spiENC28J60_SPI_MOSI_AF                   GPIO_AF_SPI2

#define spiENC28J60_CS_PIN                        GPIO_Pin_12
#define spiENC28J60_CS_GPIO_PORT                  GPIOB
#define spiENC28J60_CS_GPIO_CLK                   RCC_AHB1Periph_GPIOB

#define spiENC28J60_RESET_PIN                     GPIO_Pin_11
#define spiENC28J60_RESET_GPIO_PORT               GPIOB
#define spiENC28J60_RESET_GPIO_CLK                RCC_AHB1Periph_GPIOB


/* Exported macro ------------------------------------------------------------*/
#ifdef		USE_SOFTWAEW_SPI

#define spiENC28J60_MISO_GET() 		GPIO_ReadInputDataBit(spiENC28J60_SPI_MISO_GPIO_PORT, spiENC28J60_SPI_MISO_PIN)

/* Select SPI: SCK low */
#define spiENC28J60_SCK_LOW()       GPIO_ResetBits(spiENC28J60_SPI_SCK_GPIO_PORT, spiENC28J60_SPI_SCK_PIN)
/* Deselect SPI: SCK high */
#define spiENC28J60_SCK_HIGH()      GPIO_SetBits(spiENC28J60_SPI_SCK_GPIO_PORT, spiENC28J60_SPI_SCK_PIN)   

/* Select SPI: MOSI low */
#define spiENC28J60_MOSI_LOW()       GPIO_ResetBits(spiENC28J60_SPI_MOSI_GPIO_PORT, spiENC28J60_SPI_MOSI_PIN)
/* Deselect SPI: Reset pin high */
#define spiENC28J60_MOSI_HIGH()      GPIO_SetBits(spiENC28J60_SPI_MOSI_GPIO_PORT, spiENC28J60_SPI_MOSI_PIN)   

#endif

/* Select spiENC28J60: Chip Select pin low */
#define spiENC28J60_CS_LOW()       GPIO_ResetBits(spiENC28J60_CS_GPIO_PORT, spiENC28J60_CS_PIN)
/* Deselect spiENC28J60: Chip Select pin high */
#define spiENC28J60_CS_HIGH()      GPIO_SetBits(spiENC28J60_CS_GPIO_PORT, spiENC28J60_CS_PIN)   

/* Select spiENC28J60: RESET low */
#define spiENC28J60_RESET_LOW()       GPIO_ResetBits(spiENC28J60_RESET_GPIO_PORT, spiENC28J60_RESET_PIN)
/* Deselect SPI: Reset pin high */
#define spiENC28J60_RESET_HIGH()      GPIO_SetBits(spiENC28J60_RESET_GPIO_PORT, spiENC28J60_RESET_PIN)   


#define sFLASH_DUMMY_BYTE         0xFF


/* Exported functions ------------------------------------------------------- */

/* High layer functions  */
void spiENC28J60_DeInit(void);
void spiENC28J60_Init(void);

/* Low layer functions */
uint8_t spiENC28J60_ReadByte(void);
uint8_t spiENC28J60_SendByte(uint8_t byte);


#ifdef __cplusplus
}
#endif

#endif /* __SPI_ENC28J60_H */


/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
