/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: httpd.c,v 1.1 2001/12/12 10:00:53 adam Exp $
 */

//#include "includes.h"

#include "lwip/opt.h"
#include "lwip/def.h"
#include <string.h>


#include "lwip/debug.h"

#include "lwip/stats.h"

#include "httpd.h"

#include "lwip/tcp.h"

extern __IO uint32_t ADC3ConvertedVoltage;



#define 	_M_HTTP_DEBUG_ 		0

struct http_state {
  char *file;
  u32_t left;
};

/* This is the data for the actual web page. */
//static const char HttpHeader[] = 
static char HttpHeader[] = 
"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\
<html><head><title></title>\
<meta content=\"text/html; charset=shift_jis\" http-equiv=\"Content-Type\">\
<meta name=\"GENERATOR\" content=\"MSHTML 8.00.6001.19154\"></head>\
<body>\
<p>*************************************************<br>\
&nbsp; \
<strong>STM32F4Discovery</strong>&nbsp;&nbsp;+&nbsp; \
<strong>ENC28J60</strong>&nbsp;+&nbsp;&nbsp; \
<strong>LwIP-1.4.0<br>\
</strong>*************************************************<br>\
<br>\
&nbsp;&nbsp;&nbsp;&nbsp;<font color=\"#ff0000\">ADC3 = @@@@@@@@@@@@@@@@<br>\
</font><br>\
&nbsp;&nbsp;&nbsp; <a href=\"http://homepage2.nifty.com/yees/\">http://homepage2.nifty.com/yees/</a><br>\
<br>\
</p></body></html>";


static const char AD3tag[] = "@@@@@@@@@@@@@@@@";
int sizeADC3tag;
int positionADC3tag;

/*-----------------------------------------------------------------------------------*/
static void
conn_err(void *arg, err_t err)
{
  struct http_state *hs;

  hs = arg;
  mem_free(hs);
}
/*-----------------------------------------------------------------------------------*/
static void
close_conn(struct tcp_pcb *pcb, struct http_state *hs)
{
  tcp_arg(pcb, NULL);
  tcp_sent(pcb, NULL);
  tcp_recv(pcb, NULL);
  mem_free(hs);
  tcp_close(pcb);
}
/*-----------------------------------------------------------------------------------*/
static void
send_data(struct tcp_pcb *pcb, struct http_state *hs)
{
  err_t err;
  u16_t len;

    LWIP_DEBUGF(_M_HTTP_DEBUG_, ("httpd : send_data function entered.\n"));
  
    if(tcp_sndbuf(pcb) < hs->left) {                        /* We cannot send more data than        */    
        len = tcp_sndbuf(pcb);                              /* space avaliable in the send buffer.  */
    } else {
        len = hs->left;
    }

    err = tcp_write(pcb, hs->file, len, 0);
  
    if(err == ERR_OK) {
        hs->file += len;
        hs->left -= len;
    } else {
        LWIP_DEBUGF(_M_HTTP_DEBUG_, ("httpd : send_data error %s len %d %d\r\n", lwip_strerr(err), len, tcp_sndbuf(pcb)));
    }
}
/*-----------------------------------------------------------------------------------*/
static err_t
http_poll(void *arg, struct tcp_pcb *pcb)
{
  /*  printf("Polll\n");*/
  if(arg == NULL) {
    /*    printf("Null, close\n");*/
    tcp_close(pcb);
  } else {
    send_data(pcb, (struct http_state *)arg);
  }

  return ERR_OK;
}
/*-----------------------------------------------------------------------------------*/
static err_t http_sent(void *arg, struct tcp_pcb *pcb, u16_t len)
{
  struct http_state *hs;

    LWIP_DEBUGF(_M_HTTP_DEBUG_, ("httpd : http_sent function entered.\n"));

    hs = arg;
    if(hs->left > 0) {    
        send_data(pcb, hs);
        LWIP_DEBUGF(_M_HTTP_DEBUG_, ("httpd : http_sent data\n"));
    } else {
        close_conn(pcb, hs);
        LWIP_DEBUGF(_M_HTTP_DEBUG_, ("httpd : http_sent connection closed\n"));
    }

    return ERR_OK;
}
/*-----------------------------------------------------------------------------------*/
static err_t http_recv(void *arg, struct tcp_pcb *pcb, struct pbuf *p, err_t err)
{
  int                   i, j;
  char                  *data;
//  char                  fname[40];
//  struct    fs_file     file;
  struct    http_state  *hs;
  
  char cstr[32];

  hs = arg;

  if(err == ERR_OK && p != NULL) {

    /* Inform TCP that we have taken the data. */
    tcp_recved(pcb, p->tot_len);
    
    if(hs->file == NULL) {
        data = p->payload;
    
        LWIP_DEBUGF(_M_HTTP_DEBUG_, ("httpd : http data recieved:%s\n", data));
          
        if(strncmp(data, "GET ", 4) == 0) {
            
            LWIP_DEBUGF(_M_HTTP_DEBUG_, ("http GET::::\n"));
          
            
            pbuf_free(p);
			
			//----------------------------------------------------------------- ADC3
			for(i=0;i<sizeADC3tag;i++){
				HttpHeader[positionADC3tag + i] = ' ';
			}
			sprintf(cstr, "%lumV", ADC3ConvertedVoltage);
			j = strlen(cstr);
			if(j>sizeADC3tag){
				j = sizeADC3tag;
			}
			memcpy(&(HttpHeader[positionADC3tag]), cstr, j);
			//----------------------------------------------------------------- 
            
            hs->file = (char*)HttpHeader;
            hs->left = sizeof(HttpHeader);
            
            
            printf("httpd : start to send data %p len %ld\r\n", hs->file, hs->left);
    
            send_data(pcb, hs);
            
    
            /* Tell TCP that we wish be to informed of data that has been
                successfully sent by a call to the http_sent() function. */
            tcp_sent(pcb, http_sent);
            
            
        } else {
            close_conn(pcb, hs);
        }
    
    } else {
        pbuf_free(p);
    }
  }

  if(err == ERR_OK && p == NULL) {
    close_conn(pcb, hs);
  }
  return ERR_OK;
}
/*-----------------------------------------------------------------------------------*/
static err_t
http_accept(void *arg, struct tcp_pcb *pcb, err_t err)
{
  struct http_state *hs;

  /* Allocate memory for the structure that holds the state of the
     connection. */
  hs = mem_malloc(sizeof(struct http_state));

  if(hs == NULL) {
    printf("httpd : http_accept: Out of memory\r\n");
    return ERR_MEM;
  }
  
  /* Initialize the structure. */
  hs->file = NULL;
  hs->left = 0;

  /* Tell TCP that this is the structure we wish to be passed for our
     callbacks. */
  tcp_arg(pcb, hs);

  /* Tell TCP that we wish to be informed of incoming data by a call
     to the http_recv() function. */
  tcp_recv(pcb, http_recv);

  tcp_err(pcb, conn_err);
  
  tcp_poll(pcb, http_poll, 10);
  return ERR_OK;
}
/*-----------------------------------------------------------------------------------*/
void
httpd_init(void)
{
	struct tcp_pcb *pcb;

	pcb = tcp_new();
	tcp_bind(pcb, IP_ADDR_ANY, 80);
	pcb = tcp_listen(pcb);
	tcp_accept(pcb, http_accept);
  
	//----------------------------------------------------- ADC3
	int i, j;
	sizeADC3tag = strlen(AD3tag);
	j = sizeof(HttpHeader);
	//printf("sizeADC3tag=%d\n", sizeADC3tag);
	//printf("sizeof(HttpHeader)=%d\n", j);
	for(i=0;i<j;i++){
		if(memcmp(&(HttpHeader[i]), AD3tag, sizeADC3tag)==0){
			positionADC3tag = i;
			//printf("positionADC3tag=%d\n", positionADC3tag);
			break;
		}
	}
	//-----------------------------------------------------
  
}
/*-----------------------------------------------------------------------------------*/

