/********************************************************************************/
/*!
	@file			main.c
	@author         Nemui Trinomius (http://nemuisan.blog.bai.ne.jp)
    @version        W.I.P
    @date           2011.11.10
	@brief          BARE-METAL Project for STM32F407VGT6(STM32F4Discovery).	@n
									   and STM32F407ZGT6(REDBULL).

    @section HISTORY
		2011.11.10	W.I.P	See Whatnew.txt

    @section LICENSE
		BSD License. See Copyright.txt
*/
/********************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"
#include "platform_config.h"
#include "hw_config.h"

#include "netconf.h"
#include "helloworld.h"
#include "httpd.h"

#include "systick.h"
#include "LED.h"
#include "GLCD.h"
#include "JOY.h"
/* Defines -------------------------------------------------------------------*/
/* Variables -----------------------------------------------------------------*/

/* Constants -----------------------------------------------------------------*/
/* Function prototypes -------------------------------------------------------*/
/* Functions -----------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void System_Periodic_Handle(void);
/* Private define ------------------------------------------------------------*/
#define ADC3_DR_ADDRESS     ((uint32_t)0x4001224C)
/* Private variables ---------------------------------------------------------*/
__IO uint16_t ADC3ConvertedValue = 0;
__IO uint32_t ADC3ConvertedVoltage = 0;
/* Private functions ---------------------------------------------------------*/
void ADC3_CH12_DMA_Config(void);


/**************************************************************************/
/*! 
    @brief  Main Program.
	@param  None.
    @retval None.
*/
/**************************************************************************/
int main(void)
{
	//char *pLine;
	
	xfunc_out	= putch;
	xfunc_in	= getch;	

	/* Set Basis System */
	Set_System();
	
	/* LED Indicator */
	//LED_Configuration();
	
	/* Set SysTickCounter for _delay_ms(); */
	SysTickInit(INTERVAL);

	/* Set UART and redirect to stdio 		*/
	conio_init(UART_DEFAULT_NUM,UART_BAUDLATE);
	printf("\n");
	printf("%s test program\n",MPU_SUBMODEL);
	printf("Version %s\n",APP_VERSION);
	printf("Build Date : %s\n",__DATE__);

	printf("\n");

  /* Initilaize the LwIP satck */
	LwIP_Init();

  /* Initilaize the HelloWorld module */
	HelloWorld_init();

  /* Initilaize the webserver module */
	httpd_init();

	/* ADC3 configuration *******************************************************/
	/*  - Enable peripheral clocks                                              */
	/*  - DMA2_Stream0 channel2 configuration                                   */
	/*  - Configure ADC Channel12 pin as analog input                           */
	/*  - Configure ADC3 Channel12                                              */
	ADC3_CH12_DMA_Config();

	/* Start ADC3 Software Conversion */ 
	ADC_SoftwareStartConv(ADC3);
	
	init_get_line();
	
	
	blinkymain();
	
	/* Main Loop */
	while (1)
	{
		ADC3ConvertedVoltage = ADC3ConvertedValue *3300/0xFFF;

		/* Periodic tasks */
		System_Periodic_Handle();
	
		LwIP_Pkt_Handle();
		
		#if 0
		pLine = get_line();
		if(pLine){
			printf("%s>\n", pLine);
			
			
			init_get_line();
		}
		#endif
	}

}

/**
  * @brief  Handles the periodic tasks of the system
  * @param  None
  * @retval None
  */
void System_Periodic_Handle(void)
{
  /* Update the LCD display and the LEDs status */
  /* Manage the IP address setting */
  //Display_Periodic_Handle(LocalTime);
  
  /* LwIP periodic services are done here */
  LwIP_Periodic_Handle(LocalTime);
}

/**
  * @brief  ADC3 channel12 with DMA configuration
  * @param  None
  * @retval None
  */
void ADC3_CH12_DMA_Config(void)
{
  ADC_InitTypeDef       ADC_InitStructure;
  ADC_CommonInitTypeDef ADC_CommonInitStructure;
  DMA_InitTypeDef       DMA_InitStructure;
  GPIO_InitTypeDef      GPIO_InitStructure;

  /* Enable ADC3, DMA2 and GPIO clocks ****************************************/
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2 | RCC_AHB1Periph_GPIOC, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC3, ENABLE);

  /* DMA2 Stream0 channel0 configuration **************************************/
  DMA_InitStructure.DMA_Channel = DMA_Channel_2;  
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)ADC3_DR_ADDRESS;
  DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&ADC3ConvertedValue;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
  DMA_InitStructure.DMA_BufferSize = 1;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;         
  DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_HalfFull;
  DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
  DMA_Init(DMA2_Stream0, &DMA_InitStructure);
  DMA_Cmd(DMA2_Stream0, ENABLE);

  /* Configure ADC3 Channel12 pin as analog input ******************************/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  /* ADC Common Init **********************************************************/
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
  ADC_CommonInitStructure.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_5Cycles;
  ADC_CommonInit(&ADC_CommonInitStructure);

  /* ADC3 Init ****************************************************************/
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
  ADC_InitStructure.ADC_ScanConvMode = DISABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfConversion = 1;
  ADC_Init(ADC3, &ADC_InitStructure);

  /* ADC3 regular channel12 configuration *************************************/
  ADC_RegularChannelConfig(ADC3, ADC_Channel_12, 1, ADC_SampleTime_3Cycles);

 /* Enable DMA request after last transfer (Single-ADC mode) */
  ADC_DMARequestAfterLastTransferCmd(ADC3, ENABLE);

  /* Enable ADC3 DMA */
  ADC_DMACmd(ADC3, ENABLE);

  /* Enable ADC3 */
  ADC_Cmd(ADC3, ENABLE);
}



/* End Of File ---------------------------------------------------------------*/
