/**
  ******************************************************************************
  * @file    netconf.c
  * @author  YEES
  * @version V1.0.0
  * @date    2011/11/17
  * @brief   ENC28J60. 
  ******************************************************************************
  * @attention
  * (C) COPYRIGHT 2011 YEES 
  ******************************************************************************  
  */ 

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "lwip/opt.h"
#include "lwip/memp.h"
#include "lwip/tcp.h"
#include "lwip/tcp_impl.h"		// IwIP-1.4.0
#include "lwip/udp.h"
#include "netif/etharp.h"
#include "lwip/dhcp.h"
#include "ethernetif_enc28j60.h"
#include "netconf.h"

#include "lwip/tcpip.h"


/* Private typedef -----------------------------------------------------------*/
#define LCD_DELAY             3000
#define KEY_DELAY 			  3000
#define LCD_TIMER_MSECS       250
#define MAX_DHCP_TRIES        4
#define SELECTED              1
#define NOT_SELECTED		  (!SELECTED)
#define CLIENTMAC6            2
//#define CLIENTMAC6            3
//#define CLIENTMAC6            4

/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

struct netif gNetif;

__IO uint32_t TCPTimer = 0;
__IO uint32_t ARPTimer = 0;

#ifdef LWIP_DHCP
__IO uint32_t DHCPfineTimer = 0;
__IO uint32_t DHCPcoarseTimer = 0;
__IO uint32_t IPaddress = 0;
#endif

__IO uint32_t DisplayTimer = 0;
uint8_t LedToggle = 4;
uint8_t	Server = 0;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Initializes the lwIP stack
  * @param  None
  * @retval None
  */
void LwIP_Init(void)
{
  struct ip_addr ipaddr;
  struct ip_addr netmask;
  struct ip_addr gw;

  /* Initializes the dynamic memory heap defined by MEM_SIZE.*/
  mem_init();

  /* Initializes the memory pools defined by MEMP_NUM_x.*/
  memp_init();
  

#if LWIP_DHCP

  ipaddr.addr = 0;
  netmask.addr = 0;
  gw.addr = 0;
  
#else
	IP_ADDDRESS();
	IP_MASK();
	IP_GATEWAY();
#endif

  /* - netif_add(struct netif *netif, struct ip_addr *ipaddr,
            struct ip_addr *netmask, struct ip_addr *gw,
            void *state, err_t (* init)(struct netif *netif),
            err_t (* input)(struct pbuf *p, struct netif *netif))
    
   Adds your network interface to the netif_list. Allocate a struct
  netif and pass a pointer to this structure as the first argument.
  Give pointers to cleared ip_addr structures when using DHCP,
  or fill them with sane numbers otherwise. The state pointer may be NULL.

  The init function pointer must point to a initialization function for
  your ethernet netif interface. The following code illustrates it's use.*/
  
  if(netif_add(&gNetif, &ipaddr, &netmask, &gw, NULL, ethernetif_init, ethernet_input)==NULL){
  //if(netif_add(&gNetif, &ipaddr, &netmask, &gw, NULL, ethernetif_init, tcpip_input)==NULL){ 

	printf("ABORT: netif_add ERROR!\n");
	while(1);
  }
  
  #ifdef YDEBUG1
  printf("netif_add END   \n");	
  #endif
  
  /*  Registers the default network interface.*/
  netif_set_default(&gNetif);
  
  #ifdef YDEBUG1
  printf("netif_set_default END   \n");	
  #endif
  



#if LWIP_DHCP
  /*  Creates a new DHCP client for this interface on the first call.
  Note: you must call dhcp_fine_tmr() and dhcp_coarse_tmr() at
  the predefined regular intervals after starting the client.
  You can peek in the netif->dhcp struct for the actual DHCP status.*/
  dhcp_start(&gNetif);
#endif

  /*  When the netif is fully configured this function must be called.*/
  netif_set_up(&gNetif);
  
  #ifdef YDEBUG1
  printf("netif_set_up END   \n");	
  #endif

}

/**
  * @brief  Called when a frame is received
  * @param  None
  * @retval None
  */
void LwIP_Pkt_Handle(void)
{
  /* Read a received packet from the Ethernet buffers and send it to the lwIP for handling */
  ethernetif_input(&gNetif);
}

/**
  * @brief  LwIP periodic tasks
  * @param  localtime the current LocalTime value
  * @retval None
  */
void LwIP_Periodic_Handle(uint32_t localtime)
{

  /* TCP periodic process every 250 ms */
  if (localtime - TCPTimer >= TCP_TMR_INTERVAL)
  {
    TCPTimer =  localtime;
    tcp_tmr();
  }
  /* ARP periodic process every 5s */
  if (localtime - ARPTimer >= ARP_TMR_INTERVAL)
  {
    ARPTimer =  localtime;
    etharp_tmr();
  }

#if LWIP_DHCP
  /* Fine DHCP periodic process every 500ms */
  if (localtime - DHCPfineTimer >= DHCP_FINE_TIMER_MSECS)
  {
    DHCPfineTimer =  localtime;
    dhcp_fine_tmr();
  }
  /* DHCP Coarse periodic process every 60s */
  if (localtime - DHCPcoarseTimer >= DHCP_COARSE_TIMER_MSECS)
  {
    DHCPcoarseTimer =  localtime;
    dhcp_coarse_tmr();
  }
#endif

}

/**
  * @brief  LCD & LEDs periodic handling
  * @param  localtime: the current LocalTime value
  * @retval None
  */
  
uint8_t flagPrint = 0;		//yama

void Display_Periodic_Handle(u32_t localtime)
{ 
  /* 250 ms */
  if (localtime - DisplayTimer >= LCD_TIMER_MSECS)
  {
    DisplayTimer = localtime;

    /* We have got a new IP address so update the display */
    if (IPaddress != gNetif.ip_addr.addr)
    {
      uint8_t iptab[4];
 
      /* Read the new IP address */
      IPaddress = gNetif.ip_addr.addr;

      iptab[0] = (uint8_t)((IPaddress >> 24) & 0xFF);
      iptab[1] = (uint8_t)((IPaddress >> 16) & 0xFF);
      iptab[2] = (uint8_t)((IPaddress >> 8) & 0xFF);
      iptab[3] = (uint8_t)((IPaddress) & 0xFF);


      /* Display the new IP address */
#if LWIP_DHCP
      if (gNetif.flags & NETIF_FLAG_DHCP)
      {        
		/* Display the IP address */
		printf("IP address assigned \n");
        printf("  by a DHCP server  \n");
        printf("   %d.%d.%d.%d    \n", iptab[3], iptab[2], iptab[1], iptab[0]);
		
		iptab[0] = (uint8_t)((IPaddress >> 24) & 0xFF);
		iptab[1] = (uint8_t)((IPaddress >> 16) & 0xFF);
		iptab[2] = (uint8_t)((IPaddress >> 8) & 0xFF);
		iptab[3] = (uint8_t)((IPaddress) & 0xFF);
        	   
       
	    printf(" You are configured \n");
	    printf("is: %d.%d.%d.%d \n", iptab[3], iptab[2], iptab[1], iptab[0]);		

      }
      else
#endif
      {
        /* Display the IP address */
		printf("  Static IP address   \n");
		printf("   %d.%d.%d.%d    \n", iptab[3], iptab[2], iptab[1], iptab[0]);
      }           
    }

#if LWIP_DHCP
    
    else if (IPaddress == 0)
    {
      /* We still waiting for the DHCP server */
	  if(flagPrint==0){
	  printf("     Looking for    \n");
      printf("     DHCP server    \n");
      printf("     please wait... \n");
	  flagPrint = 1;
	  }

      LedToggle &= 3;

      STM_EVAL_LEDToggle((Led_TypeDef)(LedToggle++));

      /* If no response from a DHCP server for MAX_DHCP_TRIES times */
	  /* stop the dhcp client and set a static IP address */
	  if (gNetif.dhcp->tries > MAX_DHCP_TRIES)
      {
        struct ip_addr ipaddr;
        struct ip_addr netmask;
        struct ip_addr gw;

        printf("    DHCP timeout    \n");        

        dhcp_stop(&gNetif);

		IP_ADDDRESS();
		IP_MASK();
		IP_GATEWAY();

        netif_set_addr(&gNetif, &ipaddr , &netmask, &gw);

      }
    }
#endif
  } 
}

/******************* (C) COPYRIGHT 2011 YEES *****END OF FILE****/
